@@ -1,3 +1,12 @@
+use std::{
+    fmt::{self, Display, Formatter},
+    str::FromStr,
+};
+
+use once_cell::sync::Lazy;
+use regex::{Captures, Regex};
+
+use super::format::find_method_or_function;
 use crate::{
     analysis::object::LocationInObject,
     codegen::doc::format::{
@@ -8,14 +17,6 @@ use crate::{
     nameutil::mangle_keywords,
     Env,
 };
-use once_cell::sync::Lazy;
-use regex::{Captures, Regex};
-use std::{
-    fmt::{self, Display, Formatter},
-    str::FromStr,
-};
-
-use super::format::find_method_or_function;
 
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
 pub enum GiDocgenError {
@@ -27,10 +28,8 @@ pub enum GiDocgenError {
 impl Display for GiDocgenError {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         match self {
-            Self::InvalidLinkType(e) => f.write_str(&format!("Invalid link type \"{}\"", e)),
-            Self::BrokenLinkType(e) => {
-                f.write_str(&format!("Broken link syntax for type \"{}\"", e))
-            }
+            Self::InvalidLinkType(e) => f.write_str(&format!("Invalid link type \"{e}\"")),
+            Self::BrokenLinkType(e) => f.write_str(&format!("Broken link syntax for type \"{e}\"")),
             Self::InvalidLink => f.write_str("Invalid link syntax"),
         }
     }
@@ -58,8 +57,9 @@ fn namespace_type_from_details(
     }
 }
 
-/// Convert a "Namespace.Type.method_name" to (Option<Namespace>, Option<Type>, name)
-/// Type is only optional for global functions and the order can be modified the `is_global_func` parameters
+/// Convert a "Namespace.Type.method_name" to (Option<Namespace>, Option<Type>,
+/// name) Type is only optional for global functions and the order can be
+/// modified the `is_global_func` parameters
 fn namespace_type_method_from_details(
     link_details: &str,
     link_type: &str,
@@ -190,12 +190,41 @@ pub enum GiDocgen {
 
 fn ns_type_to_doc(namespace: &Option<String>, type_: &str) -> String {
     if let Some(ns) = namespace {
-        format!("{}::{}", ns, type_)
+        format!("{ns}::{type_}")
     } else {
         type_.to_string()
     }
 }
 
+fn find_virtual_method_by_name(
+    type_: Option<&str>,
+    namespace: Option<&str>,
+    name: &str,
+    env: &Env,
+    in_type: Option<(&TypeId, Option<LocationInObject>)>,
+) -> Option<String> {
+    find_method_or_function(
+        env,
+        in_type,
+        |f| {
+            f.name == mangle_keywords(name)
+                && namespace.as_ref().map_or(f.ns_id == MAIN_NAMESPACE, |n| {
+                    &env.library.namespaces[f.ns_id as usize].name == n
+                })
+        },
+        |o| {
+            type_.map_or(true, |t| {
+                o.name == t && is_same_namespace(env, namespace, o.type_id)
+            })
+        },
+        |_| false,
+        |_| false,
+        |_| false,
+        false,
+        true,
+    )
+}
+
 fn find_method_or_function_by_name(
     type_: Option<&str>,
     namespace: Option<&str>,
@@ -205,7 +234,6 @@ fn find_method_or_function_by_name(
     is_class_method: bool,
 ) -> Option<String> {
     find_method_or_function(
-        name,
         env,
         in_type,
         |f| {
@@ -235,6 +263,7 @@ fn find_method_or_function_by_name(
             })
         },
         is_class_method,
+        false,
     )
 }
 
@@ -331,7 +360,7 @@ impl GiDocgen {
                     },
                 ),
             GiDocgen::Id(c_name) => symbols.by_c_name(c_name).map_or_else(
-                || format!("`{}`", c_name),
+                || format!("`{c_name}`"),
                 |sym| format!("[`{n}`][crate::{n}]", n = sym.full_rust_name()),
             ),
             GiDocgen::Struct { namespace, type_ } => env
@@ -378,7 +407,7 @@ impl GiDocgen {
                 if let Some(ty) = type_ {
                     format!("`{}::{}()`", ns_type_to_doc(namespace, ty), name)
                 } else {
-                    format!("`{}()`", name)
+                    format!("`{name}()`")
                 }
             }),
             GiDocgen::Alias(alias) => gen_alias_doc_link(alias),
@@ -403,7 +432,8 @@ impl GiDocgen {
                 namespace,
                 type_,
                 name,
-            } => gen_vfunc_doc_link(&ns_type_to_doc(namespace, type_), name),
+            } => find_virtual_method_by_name(Some(type_), namespace.as_deref(), name, env, in_type)
+                .unwrap_or_else(|| gen_vfunc_doc_link(&ns_type_to_doc(namespace, type_), name)),
         }
     }
 }
@@ -415,19 +445,19 @@ impl FromStr for GiDocgen {
         let item_link = item_link.trim_start_matches('[').trim_end_matches(']');
         if let Some((link_type, link_details)) = item_link.split_once('@') {
             match link_type {
-                "alias" => Ok(GiDocgen::Alias(link_details.to_string())),
+                "alias" => Ok(Self::Alias(link_details.to_string())),
                 "class" => {
                     let (namespace, type_) = namespace_type_from_details(link_details, "class")?;
-                    Ok(GiDocgen::Class { namespace, type_ })
+                    Ok(Self::Class { namespace, type_ })
                 }
                 "const" => {
                     let (namespace, type_) = namespace_type_from_details(link_details, "const")?;
-                    Ok(GiDocgen::Const { namespace, type_ })
+                    Ok(Self::Const { namespace, type_ })
                 }
                 "ctor" => {
                     let (namespace, type_, name) =
                         namespace_type_method_from_details(link_details, "ctor", false)?;
-                    Ok(GiDocgen::Constructor {
+                    Ok(Self::Constructor {
                         namespace,
                         type_: type_
                             .ok_or_else(|| GiDocgenError::BrokenLinkType("ctor".to_string()))?,
@@ -436,20 +466,20 @@ impl FromStr for GiDocgen {
                 }
                 "enum" => {
                     let (namespace, type_) = namespace_type_from_details(link_details, "enum")?;
-                    Ok(GiDocgen::Enum { namespace, type_ })
+                    Ok(Self::Enum { namespace, type_ })
                 }
                 "error" => {
                     let (namespace, type_) = namespace_type_from_details(link_details, "error")?;
-                    Ok(GiDocgen::Error { namespace, type_ })
+                    Ok(Self::Error { namespace, type_ })
                 }
                 "flags" => {
                     let (namespace, type_) = namespace_type_from_details(link_details, "flags")?;
-                    Ok(GiDocgen::Flag { namespace, type_ })
+                    Ok(Self::Flag { namespace, type_ })
                 }
                 "func" => {
                     let (namespace, type_, name) =
                         namespace_type_method_from_details(link_details, "func", true)?;
-                    Ok(GiDocgen::Func {
+                    Ok(Self::Func {
                         namespace,
                         type_,
                         name,
@@ -457,18 +487,18 @@ impl FromStr for GiDocgen {
                 }
                 "iface" => {
                     let (namespace, type_) = namespace_type_from_details(link_details, "iface")?;
-                    Ok(GiDocgen::Interface { namespace, type_ })
+                    Ok(Self::Interface { namespace, type_ })
                 }
                 "callback" => {
                     let (namespace, name) = namespace_type_from_details(link_details, "callback")?;
-                    Ok(GiDocgen::Callback { namespace, name })
+                    Ok(Self::Callback { namespace, name })
                 }
                 "method" => {
                     let (namespace, type_, name) =
                         namespace_type_method_from_details(link_details, "method", false)?;
                     let type_ =
                         type_.ok_or_else(|| GiDocgenError::BrokenLinkType("method".to_string()))?;
-                    Ok(GiDocgen::Method {
+                    Ok(Self::Method {
                         namespace,
                         is_class_method: type_.ends_with("Class"),
                         type_,
@@ -481,7 +511,7 @@ impl FromStr for GiDocgen {
                     if type_details.len() < 2 || type_details[1].is_empty() {
                         Err(GiDocgenError::BrokenLinkType("property".to_string()))
                     } else {
-                        Ok(GiDocgen::Property {
+                        Ok(Self::Property {
                             namespace,
                             type_: type_details[0].to_string(),
                             name: type_details[1].to_string(),
@@ -494,7 +524,7 @@ impl FromStr for GiDocgen {
                     if type_details.len() < 2 || type_details[1].is_empty() {
                         Err(GiDocgenError::BrokenLinkType("signal".to_string()))
                     } else {
-                        Ok(GiDocgen::Signal {
+                        Ok(Self::Signal {
                             namespace,
                             type_: type_details[0].to_string(),
                             name: type_details[1].to_string(),
@@ -503,19 +533,19 @@ impl FromStr for GiDocgen {
                 }
                 "struct" => {
                     let (namespace, type_) = namespace_type_from_details(link_details, "struct")?;
-                    Ok(GiDocgen::Struct { namespace, type_ })
+                    Ok(Self::Struct { namespace, type_ })
                 }
                 "vfunc" => {
                     let (namespace, type_, name) =
                         namespace_type_method_from_details(link_details, "vfunc", false)?;
-                    Ok(GiDocgen::VFunc {
+                    Ok(Self::VFunc {
                         namespace,
                         type_: type_
                             .ok_or_else(|| GiDocgenError::BrokenLinkType("vfunc".to_string()))?,
                         name,
                     })
                 }
-                "id" => Ok(GiDocgen::Id(link_details.to_string())),
+                "id" => Ok(Self::Id(link_details.to_string())),
                 e => Err(GiDocgenError::InvalidLinkType(e.to_string())),
             }
         } else {
