@@ -1,3 +1,14 @@
+use std::{
+    borrow::Cow,
+    collections::{BTreeSet, HashSet},
+    io::{Result, Write},
+};
+
+use log::{error, info};
+use once_cell::sync::Lazy;
+use regex::{Captures, Regex};
+use stripper_lib::{write_file_name, write_item_doc, Type as SType, TypeStruct};
+
 use self::format::reformat_doc;
 use crate::{
     analysis::{self, namespaces::MAIN, object::LocationInObject},
@@ -9,15 +20,6 @@ use crate::{
     traits::*,
     version::Version,
 };
-use log::{error, info};
-use once_cell::sync::Lazy;
-use regex::{Captures, Regex};
-use std::{
-    borrow::Cow,
-    collections::{BTreeSet, HashSet},
-    io::{Result, Write},
-};
-use stripper_lib::{write_file_name, write_item_doc, Type as SType, TypeStruct};
 
 mod format;
 mod gi_docgen;
@@ -252,6 +254,7 @@ fn create_object_doc(w: &mut dyn Write, env: &Env, info: &analysis::object::Info
     let doc;
     let doc_deprecated;
     let functions: &[Function];
+    let virtual_methods: &[Function];
     let signals: &[Signal];
     let properties: &[Property];
     let is_abstract;
@@ -268,6 +271,7 @@ fn create_object_doc(w: &mut dyn Write, env: &Env, info: &analysis::object::Info
             doc = cl.doc.as_ref();
             doc_deprecated = cl.doc_deprecated.as_ref();
             functions = &cl.functions;
+            virtual_methods = &cl.virtual_methods;
             signals = &cl.signals;
             properties = &cl.properties;
             is_abstract = env.library.type_(info.type_id).is_abstract();
@@ -277,6 +281,7 @@ fn create_object_doc(w: &mut dyn Write, env: &Env, info: &analysis::object::Info
             doc = iface.doc.as_ref();
             doc_deprecated = iface.doc_deprecated.as_ref();
             functions = &iface.functions;
+            virtual_methods = &iface.virtual_methods;
             signals = &iface.signals;
             properties = &iface.properties;
             is_abstract = false;
@@ -319,6 +324,53 @@ fn create_object_doc(w: &mut dyn Write, env: &Env, info: &analysis::object::Info
             )?;
         }
 
+        if !properties.is_empty() {
+            writeln!(w, "\n## Properties")?;
+            document_type_properties(env, w, info, properties, None)?;
+
+            for parent_info in &info.supertypes {
+                match env.library.type_(parent_info.type_id) {
+                    Type::Class(cl) => {
+                        if !cl.properties.is_empty() {
+                            document_type_properties(env, w, info, &cl.properties, Some(&cl.name))?;
+                        }
+                    }
+                    Type::Interface(iface) => {
+                        if !iface.properties.is_empty() {
+                            document_type_properties(
+                                env,
+                                w,
+                                info,
+                                &iface.properties,
+                                Some(&iface.name),
+                            )?;
+                        }
+                    }
+                    _ => (),
+                }
+            }
+        }
+        if !signals.is_empty() {
+            writeln!(w, "\n## Signals")?;
+            document_type_signals(env, w, info, signals, None)?;
+
+            for parent_info in &info.supertypes {
+                match env.library.type_(parent_info.type_id) {
+                    Type::Class(cl) => {
+                        if !cl.signals.is_empty() {
+                            document_type_signals(env, w, info, &cl.signals, Some(&cl.name))?;
+                        }
+                    }
+                    Type::Interface(iface) => {
+                        if !iface.signals.is_empty() {
+                            document_type_signals(env, w, info, &iface.signals, Some(&iface.name))?;
+                        }
+                    }
+                    _ => (),
+                }
+            }
+        }
+
         let impl_self = if has_trait { Some(info.type_id) } else { None };
         let mut implements = impl_self
             .iter()
@@ -341,7 +393,6 @@ fn create_object_doc(w: &mut dyn Write, env: &Env, info: &analysis::object::Info
 
     if has_builder {
         let builder_ty = TypeStruct::new(SType::Impl, &format!("{}Builder", info.name));
-
         let mut builder_properties: Vec<_> = properties.iter().collect();
         for parent_info in &info.supertypes {
             match env.library.type_(parent_info.type_id) {
@@ -507,6 +558,56 @@ fn create_object_doc(w: &mut dyn Write, env: &Env, info: &analysis::object::Info
             configured_signals.iter().all(|s| s.generate_doc),
         )?;
     }
+
+    for function in virtual_methods {
+        let configured_virtual_methods = obj.virtual_methods.matched(&function.name);
+        let (ty, object_location) = if let Some(trait_name) = configured_virtual_methods
+            .iter()
+            .find_map(|f| f.doc_trait_name.as_ref())
+        {
+            (
+                TypeStruct::new(SType::Trait, trait_name),
+                // Because we cannot sensibly deduce where the docs end up,
+                // assume they're outside the docs so that no `Self::` links
+                // are generated.  It is currently quite uncommon to specify
+                // the `{}Manual` trait, which would be ObjectLocation::ExtManual.
+                None,
+            )
+        } else {
+            (
+                TypeStruct::new(SType::Trait, &format!("{}Impl", info.name)),
+                Some(LocationInObject::VirtualExt),
+            )
+        };
+
+        if let Some(c_identifier) = &function.c_identifier {
+            let f_info: Option<&analysis::functions::Info> = info
+                .virtual_methods
+                .iter()
+                .find(|f| &f.glib_name == c_identifier);
+            let should_be_documented = f_info.map_or(false, |f| f.should_docs_be_generated(env));
+            if !should_be_documented {
+                continue;
+            }
+
+            // Retrieve the new_name computed during analysis, if any
+            let fn_new_name = f_info.and_then(|analysed_f| analysed_f.new_name.clone());
+            let doc_ignored_parameters = f_info
+                .map(|analyzed_f| analyzed_f.doc_ignore_parameters.clone())
+                .unwrap_or_default();
+            create_fn_doc(
+                w,
+                env,
+                function,
+                Some(Box::new(ty)),
+                fn_new_name,
+                &doc_ignored_parameters,
+                Some((&info.type_id, object_location)),
+                f_info.map_or(true, |f| f.generate_doc),
+            )?;
+        }
+    }
+
     for property in properties {
         let getter_name = info
             .properties
@@ -551,19 +652,18 @@ fn create_object_doc(w: &mut dyn Write, env: &Env, info: &analysis::object::Info
 fn create_record_doc(w: &mut dyn Write, env: &Env, info: &analysis::record::Info) -> Result<()> {
     let record: &Record = env.library.type_(info.type_id).to_ref_as();
     let ty = record.to_stripper_type();
-    let generate_doc = env
-        .config
-        .objects
-        .get(&info.type_id.full_name(&env.library))
-        .map_or(true, |r| r.generate_doc);
-
+    let object = env.config.objects.get(&info.full_name);
+    let trait_name = object
+        .and_then(|o| o.trait_name.clone())
+        .unwrap_or_else(|| format!("{}Ext", info.name));
+    let generate_doc = object.map_or(true, |r| r.generate_doc);
     if generate_doc {
         write_item_doc(w, &ty, |w| {
             if let Some(ref doc) = record.doc {
                 writeln!(w, "{}", reformat_doc(doc, env, Some((&info.type_id, None))))?;
             }
             if let Some(ver) = info.deprecated_version {
-                writeln!(w, "\n# Deprecated since {}\n", ver)?;
+                writeln!(w, "\n# Deprecated since {ver}\n")?;
             } else if record.doc_deprecated.is_some() {
                 writeln!(w, "\n# Deprecated\n")?;
             }
@@ -574,11 +674,17 @@ fn create_record_doc(w: &mut dyn Write, env: &Env, info: &analysis::record::Info
         })?;
     }
 
-    let ty = TypeStruct {
-        ty: SType::Impl,
-        ..ty
-    };
     for function in &record.functions {
+        let function_ty = if function.kind == FunctionKind::ClassMethod {
+            TypeStruct::new(SType::Trait, &trait_name)
+        } else {
+            TypeStruct {
+                ty: SType::Impl,
+                parent: ty.parent.clone(),
+                name: ty.name.clone(),
+                args: ty.args.clone(),
+            }
+        };
         if let Some(c_identifier) = &function.c_identifier {
             let f_info = info.functions.iter().find(|f| &f.glib_name == c_identifier);
             let should_be_documented = f_info.map_or(false, |f| f.should_docs_be_generated(env));
@@ -591,7 +697,7 @@ fn create_record_doc(w: &mut dyn Write, env: &Env, info: &analysis::record::Info
                 w,
                 env,
                 function,
-                Some(Box::new(ty.clone())),
+                Some(Box::new(function_ty)),
                 fn_new_name,
                 &HashSet::new(),
                 Some((&info.type_id, None)),
@@ -612,7 +718,7 @@ fn create_enum_doc(w: &mut dyn Write, env: &Env, enum_: &Enumeration, tid: TypeI
                 writeln!(w, "{}", reformat_doc(doc, env, Some((&tid, None))))?;
             }
             if let Some(ver) = enum_.deprecated_version {
-                writeln!(w, "\n# Deprecated since {}\n", ver)?;
+                writeln!(w, "\n# Deprecated since {ver}\n")?;
             } else if enum_.doc_deprecated.is_some() {
                 writeln!(w, "\n# Deprecated\n")?;
             }
@@ -671,7 +777,7 @@ fn create_bitfield_doc(
             }
         }
         if let Some(ver) = bitfield.deprecated_version {
-            writeln!(w, "\n# Deprecated since {}\n", ver)?;
+            writeln!(w, "\n# Deprecated since {ver}\n")?;
         } else if bitfield.doc_deprecated.is_some() {
             writeln!(w, "\n# Deprecated\n")?;
         }
@@ -773,7 +879,7 @@ where
             )?;
         }
         if let Some(ver) = fn_.deprecated_version() {
-            writeln!(w, "\n# Deprecated since {}\n", ver)?;
+            writeln!(w, "\n# Deprecated since {ver}\n")?;
         } else if fn_.doc_deprecated().is_some() {
             writeln!(w, "\n# Deprecated\n")?;
         }
@@ -937,7 +1043,7 @@ fn create_property_doc(
                 )?;
             }
             if let Some(ver) = property.deprecated_version {
-                writeln!(w, "\n# Deprecated since {}\n", ver)?;
+                writeln!(w, "\n# Deprecated since {ver}\n")?;
             } else if property.doc_deprecated.is_some() {
                 writeln!(w, "\n# Deprecated\n")?;
             }
@@ -965,14 +1071,14 @@ fn get_type_trait_for_implements(env: &Env, tid: TypeId) -> String {
         format!("{}Ext", env.library.type_(tid).get_name())
     };
     if tid.ns_id == MAIN_NAMESPACE {
-        format!("[`{0}`][trait@crate::prelude::{0}]", trait_name)
+        format!("[`{trait_name}`][trait@crate::prelude::{trait_name}]")
     } else if let Some(symbol) = env.symbols.borrow().by_tid(tid) {
         let mut symbol = symbol.clone();
         symbol.make_trait(&trait_name);
         format!("[`trait@{}`]", &symbol.full_rust_name())
     } else {
         error!("Type {} doesn't have crate", tid.full_name(&env.library));
-        format!("`{}`", trait_name)
+        format!("`{trait_name}`")
     }
 }
 
@@ -996,6 +1102,92 @@ pub fn get_type_manual_traits_for_implements(
     manual_trait_iters
         .into_iter()
         .flatten()
-        .map(|name| format!("[`{0}`][trait@crate::prelude::{0}]", name))
+        .map(|name| format!("[`{name}`][trait@crate::prelude::{name}]"))
         .collect()
 }
+
+pub fn document_type_properties(
+    env: &Env,
+    w: &mut dyn Write,
+    info: &analysis::object::Info,
+    properties: &[Property],
+    subtype: Option<&str>,
+) -> Result<()> {
+    if let Some(subtype_name) = subtype {
+        writeln!(w, "<details><summary><h4>{subtype_name}</h4></summary>")?;
+    }
+    for property in properties {
+        let mut details = Vec::new();
+        if property.readable {
+            details.push("Readable");
+        }
+        if property.writable {
+            details.push("Writeable");
+        }
+        if property.construct {
+            details.push("Construct");
+        }
+        if property.construct_only {
+            details.push("Construct Only");
+        }
+        if let Some(doc) = &property.doc {
+            writeln!(
+                w,
+                "\n\n#### `{}`\n {}\n\n{}",
+                property.name,
+                reformat_doc(
+                    &fix_param_names(doc, &None),
+                    env,
+                    Some((&info.type_id, None))
+                ),
+                details.join(" | "),
+            )?;
+        } else {
+            writeln!(w, "\n\n#### `{}`\n {}", property.name, details.join(" | "),)?;
+        }
+    }
+    if subtype.is_some() {
+        writeln!(w, "</details>")?;
+    }
+    Ok(())
+}
+
+pub fn document_type_signals(
+    env: &Env,
+    w: &mut dyn Write,
+    info: &analysis::object::Info,
+    signals: &[Signal],
+    subtype: Option<&str>,
+) -> Result<()> {
+    if let Some(subtype_name) = subtype {
+        writeln!(w, "<details><summary><h4>{subtype_name}</h4></summary>")?;
+    }
+    for signal in signals {
+        let mut details = Vec::new();
+        if signal.is_action {
+            details.push("Action");
+        }
+        if signal.is_detailed {
+            details.push("Detailed");
+        }
+        if let Some(doc) = &signal.doc {
+            writeln!(
+                w,
+                "\n\n#### `{}`\n {}\n\n{}",
+                signal.name,
+                reformat_doc(
+                    &fix_param_names(doc, &None),
+                    env,
+                    Some((&info.type_id, None))
+                ),
+                details.join(" | "),
+            )?;
+        } else {
+            writeln!(w, "\n\n#### `{}`\n {}", signal.name, details.join(" | "),)?;
+        }
+    }
+    if subtype.is_some() {
+        writeln!(w, "</details>")?;
+    }
+    Ok(())
+}
