@@ -1,3 +1,5 @@
+use std::collections::{hash_map::Entry, BTreeMap, HashMap};
+
 use crate::{
     analysis::{
         self,
@@ -19,11 +21,10 @@ use crate::{
     nameutil::{is_gstring, use_gio_type, use_glib_if_needed, use_glib_type},
     traits::*,
 };
-use std::collections::{hash_map::Entry, BTreeMap, HashMap};
 
 #[derive(Clone, Debug)]
 enum Parameter {
-    //Used to separate in and out parameters in `add_in_array_lengths`
+    // Used to separate in and out parameters in `add_in_array_lengths`
     // and `generate_func_parameters`
     In,
     Out {
@@ -43,7 +44,7 @@ enum OutMemMode {
 
 impl OutMemMode {
     fn is_uninitialized(&self) -> bool {
-        matches!(*self, OutMemMode::Uninitialized)
+        matches!(*self, Self::Uninitialized)
     }
 }
 
@@ -78,38 +79,38 @@ struct FuncParameter<'a> {
 }
 
 impl Builder {
-    pub fn new() -> Builder {
+    pub fn new() -> Self {
         Default::default()
     }
-    pub fn async_trampoline(&mut self, trampoline: &AsyncTrampoline) -> &mut Builder {
+    pub fn async_trampoline(&mut self, trampoline: &AsyncTrampoline) -> &mut Self {
         self.async_trampoline = Some(trampoline.clone());
         self
     }
-    pub fn callback(&mut self, trampoline: &Trampoline) -> &mut Builder {
+    pub fn callback(&mut self, trampoline: &Trampoline) -> &mut Self {
         self.callbacks.push(trampoline.clone());
         self
     }
-    pub fn destroy(&mut self, trampoline: &Trampoline) -> &mut Builder {
+    pub fn destroy(&mut self, trampoline: &Trampoline) -> &mut Self {
         self.destroys.push(trampoline.clone());
         self
     }
-    pub fn glib_name(&mut self, name: &str) -> &mut Builder {
+    pub fn glib_name(&mut self, name: &str) -> &mut Self {
         self.glib_name = name.into();
         self
     }
-    pub fn assertion(&mut self, assertion: SafetyAssertionMode) -> &mut Builder {
+    pub fn assertion(&mut self, assertion: SafetyAssertionMode) -> &mut Self {
         self.assertion = assertion;
         self
     }
-    pub fn ret(&mut self, ret: &return_value::Info) -> &mut Builder {
+    pub fn ret(&mut self, ret: &return_value::Info) -> &mut Self {
         self.ret = ReturnValue { ret: ret.clone() };
         self
     }
-    pub fn parameter(&mut self) -> &mut Builder {
+    pub fn parameter(&mut self) -> &mut Self {
         self.parameters.push(Parameter::In);
         self
     }
-    pub fn out_parameter(&mut self, env: &Env, parameter: &AnalysisCParameter) -> &mut Builder {
+    pub fn out_parameter(&mut self, env: &Env, parameter: &AnalysisCParameter) -> &mut Self {
         let mem_mode = c_type_mem_mode(env, parameter);
         self.parameters.push(Parameter::Out {
             parameter: parameter_ffi_call_out::Parameter::new(
@@ -122,16 +123,16 @@ impl Builder {
         self
     }
 
-    pub fn transformations(&mut self, transformations: &[Transformation]) -> &mut Builder {
+    pub fn transformations(&mut self, transformations: &[Transformation]) -> &mut Self {
         self.transformations = transformations.to_owned();
         self
     }
 
-    pub fn outs_mode(&mut self, mode: Mode) -> &mut Builder {
+    pub fn outs_mode(&mut self, mode: Mode) -> &mut Self {
         self.outs_mode = mode;
         self
     }
-    pub fn in_unsafe(&mut self, in_unsafe: bool) -> &mut Builder {
+    pub fn in_unsafe(&mut self, in_unsafe: bool) -> &mut Self {
         self.in_unsafe = in_unsafe;
         self
     }
@@ -271,7 +272,7 @@ impl Builder {
             {
                 if calls.len() > 1 {
                     chunks.push(Chunk::Let {
-                        name: format!("super_callback{}", pos),
+                        name: format!("super_callback{pos}"),
                         is_mut: false,
                         value: Box::new(Chunk::Custom(if poses.is_empty() {
                             format!(
@@ -307,7 +308,7 @@ impl Builder {
                     });
                 } else if !calls.is_empty() {
                     chunks.push(Chunk::Let {
-                        name: format!("super_callback{}", pos),
+                        name: format!("super_callback{pos}"),
                         is_mut: false,
                         value: Box::new(Chunk::Custom(format!(
                             "{}{}_data",
@@ -454,13 +455,12 @@ impl Builder {
             .last()
             .map_or_else(|| "Unknown".to_owned(), |p| p.name.clone());
 
-        let mut extra_before_call = "";
         if let Some(full_type) = full_type {
             if is_destroy || trampoline.scope.is_async() {
                 body.push(Chunk::Let {
                     name: format!("{}callback", if is_destroy { "_" } else { "" }),
                     is_mut: false,
-                    value: Box::new(Chunk::Custom(format!("Box_::from_raw({} as *mut _)", func))),
+                    value: Box::new(Chunk::Custom(format!("Box_::from_raw({func} as *mut _)"))),
                     type_: Some(Box::new(Chunk::Custom(full_type.1.clone()))),
                 });
             } else {
@@ -490,7 +490,7 @@ impl Builder {
                     body.push(Chunk::Custom(format!(
                         "let callback = callback{}{};",
                         if let Some(pos) = pos {
-                            format!(".{}", pos)
+                            format!(".{pos}")
                         } else {
                             String::new()
                         },
@@ -500,20 +500,12 @@ impl Builder {
                             ""
                         }
                     )));
-                    if trampoline.ret.c_type != "void" {
-                        extra_before_call = "let res = ";
-                    }
                 } else if !trampoline.scope.is_call() {
                     if *trampoline.nullable {
                         body.push(Chunk::Custom(format!(
-                            "{}if let Some(ref callback) = callback{} {{",
-                            if trampoline.ret.c_type != "void" {
-                                "let res = "
-                            } else {
-                                ""
-                            },
+                            "if let Some(ref callback) = callback{} {{",
                             if let Some(pos) = pos {
-                                format!(".{}", pos)
+                                format!(".{pos}")
                             } else {
                                 String::new()
                             }
@@ -522,39 +514,26 @@ impl Builder {
                         body.push(Chunk::Custom(format!(
                             "let callback = callback{}",
                             if let Some(pos) = pos {
-                                format!(".{}", pos)
+                                format!(".{pos}")
                             } else {
                                 String::new()
                             }
                         )));
-                        if trampoline.ret.c_type != "void" {
-                            body.push(Chunk::Custom("let res = ".to_owned()));
+                    }
+                } else if !trampoline.scope.is_async() && *trampoline.nullable {
+                    body.push(Chunk::Custom(format!(
+                        "if let Some(ref {}callback) = {} {{",
+                        if trampoline.scope.is_call() {
+                            "mut "
+                        } else {
+                            ""
+                        },
+                        if let Some(pos) = pos {
+                            format!("(*callback).{pos}")
+                        } else {
+                            "*callback".to_owned()
                         }
-                    }
-                } else {
-                    let add = if trampoline.ret.c_type != "void" {
-                        "let res = "
-                    } else {
-                        ""
-                    };
-                    if !trampoline.scope.is_async() && *trampoline.nullable {
-                        body.push(Chunk::Custom(format!(
-                            "{}if let Some(ref {}callback) = {} {{",
-                            add,
-                            if trampoline.scope.is_call() {
-                                "mut "
-                            } else {
-                                ""
-                            },
-                            if let Some(pos) = pos {
-                                format!("(*callback).{}", pos)
-                            } else {
-                                "*callback".to_owned()
-                            }
-                        )));
-                    } else {
-                        body.push(Chunk::Custom(add.to_owned()));
-                    }
+                    )));
                 }
             }
         } else {
@@ -563,14 +542,14 @@ impl Builder {
                 is_mut: false,
                 value: Box::new(Chunk::Custom(
                     if is_destroy || trampoline.scope.is_async() {
-                        format!("Box_::from_raw({} as *mut _)", func)
+                        format!("Box_::from_raw({func} as *mut _)")
                     } else if trampoline.scope.is_call() {
                         format!(
                             "{} as *const _ as usize as *mut {}",
                             func, trampoline.bound_name
                         )
                     } else {
-                        format!("&*({} as *mut _)", func)
+                        format!("&*({func} as *mut _)")
                     },
                 )),
                 type_: Some(Box::new(Chunk::Custom(
@@ -588,38 +567,27 @@ impl Builder {
                     body.push(Chunk::Custom(
                         "let callback = (*callback).expect(\"cannot get closure...\");".to_owned(),
                     ));
-                    if trampoline.ret.c_type != "void" {
-                        extra_before_call = "let res = ";
-                    }
                 } else {
                     body.push(Chunk::Custom(format!(
-                        "{}if let Some(ref {}callback) = {} {{",
-                        if trampoline.ret.c_type != "void" {
-                            "let res = "
-                        } else {
-                            ""
-                        },
+                        "if let Some(ref {}callback) = {} {{",
                         if trampoline.scope.is_call() {
                             "mut "
                         } else {
                             ""
                         },
                         if let Some(pos) = pos {
-                            format!("(*callback).{}", pos)
+                            format!("(*callback).{pos}")
                         } else {
                             "*callback".to_owned()
                         }
                     )));
                 }
-            } else if !is_destroy && trampoline.ret.c_type != "void" {
-                extra_before_call = "let res = ";
             }
         }
         if !is_destroy {
             use crate::writer::to_code::ToCode;
             body.push(Chunk::Custom(format!(
-                "{}{}({}){}",
-                extra_before_call,
+                "{}({})",
                 if !*trampoline.nullable {
                     "(*callback)"
                 } else if trampoline.scope.is_async() {
@@ -632,26 +600,18 @@ impl Builder {
                     .flat_map(|arg| arg.to_code(env))
                     .collect::<Vec<_>>()
                     .join(", "),
-                if !extra_before_call.is_empty() || !*trampoline.nullable {
-                    ";"
-                } else {
-                    ""
-                }
             )));
             if !trampoline.scope.is_async() && *trampoline.nullable {
                 body.push(Chunk::Custom("} else {".to_owned()));
                 body.push(Chunk::Custom(
                     "\tpanic!(\"cannot get closure...\")".to_owned(),
                 ));
-                body.push(Chunk::Custom("};".to_owned()));
+                body.push(Chunk::Custom("}".to_owned()));
             }
             if trampoline.ret.c_type != "void" {
                 use crate::codegen::trampoline_to_glib::TrampolineToGlib;
 
-                body.push(Chunk::Custom(format!(
-                    "res{}",
-                    trampoline.ret.trampoline_to_glib(env)
-                )));
+                body.push(Chunk::Custom(trampoline.ret.trampoline_to_glib(env)));
             }
         }
 
@@ -696,7 +656,7 @@ impl Builder {
         let bounds_str = if bounds_names.is_empty() {
             String::new()
         } else {
-            format!("::<{}>", bounds_names)
+            format!("::<{bounds_names}>")
         };
         if !is_destroy {
             if *trampoline.nullable {
@@ -848,14 +808,25 @@ impl Builder {
                 );
             }
         }
+        // If the trampoline doesn't have a GError parameter
+        let has_error_parameter = self
+            .async_trampoline
+            .as_ref()
+            .map(|a| a.has_error_parameter)
+            .unwrap_or_default();
 
         let result = Chunk::Tuple(result, TupleMode::WithUnit);
-        let mut body = vec![Chunk::Let {
-            name: "error".to_string(),
-            is_mut: true,
-            value: Box::new(Chunk::NullMutPtr),
-            type_: None,
-        }];
+
+        let mut body = if has_error_parameter {
+            vec![Chunk::Let {
+                name: "error".to_string(),
+                is_mut: true,
+                value: Box::new(Chunk::NullMutPtr),
+                type_: None,
+            }]
+        } else {
+            vec![]
+        };
         let output_vars = trampoline
             .output_params
             .iter()
@@ -871,7 +842,13 @@ impl Builder {
         body.extend(output_vars);
 
         let ret_name = if trampoline.ffi_ret.is_some() {
-            "ret"
+            if has_error_parameter {
+                "ret"
+            } else {
+                "result" // Needed as in case of an error param we would have
+                         // let result = if error.is_null() { Ok()} else {
+                         // Err()};
+            }
         } else {
             "_"
         };
@@ -885,15 +862,17 @@ impl Builder {
             }),
             type_: None,
         });
-        body.push(Chunk::Let {
-            name: "result".to_string(),
-            is_mut: false,
-            value: Box::new(Chunk::ErrorResultReturn {
-                ret: None,
-                value: Box::new(result),
-            }),
-            type_: None,
-        });
+        if has_error_parameter {
+            body.push(Chunk::Let {
+                name: "result".to_string(),
+                is_mut: false,
+                value: Box::new(Chunk::ErrorResultReturn {
+                    ret: None,
+                    value: Box::new(result),
+                }),
+                type_: None,
+            });
+        }
         body.push(Chunk::Let {
             name: "callback".to_string(),
             is_mut: false,
@@ -994,11 +973,10 @@ impl Builder {
 
     fn generate_call(&self, calls: &FuncParameters<'_>) -> Chunk {
         let params = self.generate_func_parameters(calls);
-        let func = Chunk::FfiCall {
+        Chunk::FfiCall {
             name: self.glib_name.clone(),
             params,
-        };
-        func
+        }
     }
     fn generate_call_conversion(
         &self,
@@ -1038,9 +1016,9 @@ impl Builder {
                 Chunk::FfiCallParameter {
                     transformation_type: TransformationType::ToGlibDirect {
                         name: if all_call {
-                            format!("super_callback{} as *const _ as usize as *mut _", pos)
+                            format!("super_callback{pos} as *const _ as usize as *mut _")
                         } else {
-                            format!("Box_::into_raw(super_callback{}) as *mut _", pos)
+                            format!("Box_::into_raw(super_callback{pos}) as *mut _")
                         },
                     },
                 },
@@ -1081,7 +1059,8 @@ impl Builder {
             .collect()
     }
     fn check_if_need_glib_conversion(&self, env: &Env, typ: TypeId) -> bool {
-        // TODO: maybe improve this part to potentially handle more cases than just glib::Pid?
+        // TODO: maybe improve this part to potentially handle more cases than just
+        // glib::Pid?
         matches!(
             env.type_(typ),
             library::Type::Alias(a) if a.c_identifier == "GPid"
@@ -1123,7 +1102,6 @@ impl Builder {
         }
     }
 
-    #[allow(clippy::blocks_in_if_conditions)]
     fn generate_out_return(&self, uninitialized_vars: &mut Vec<(String, bool)>) -> Option<Chunk> {
         if !self.outs_as_return {
             return None;
@@ -1211,7 +1189,7 @@ impl Builder {
                 (call, Some(ret))
             }
             Throws(return_strategy) => {
-                //extracting original FFI function call
+                // extracting original FFI function call
                 let (boxed_call, array_length_name, ret_info) = if let Chunk::FfiCallConversion {
                     call: inner,
                     array_length_name,
@@ -1403,9 +1381,9 @@ fn add_chunk_for_type(
                 }
             } else if par.transfer == library::Transfer::None && *nullable {
                 if par.conversion_type == ConversionType::Borrow {
-                    type_name = format!(": Borrowed<Option<{}>>", ty_name);
+                    type_name = format!(": Borrowed<Option<{ty_name}>>");
                 } else {
-                    type_name = format!(": Option<{}>", ty_name);
+                    type_name = format!(": Option<{ty_name}>");
                 }
             } else {
                 type_name = String::new();
