@@ -1,3 +1,8 @@
+use std::{collections::HashSet, str::FromStr};
+
+use log::error;
+use toml::Value;
+
 use super::{
     error::TomlHelper,
     gobjects::GStatus,
@@ -12,9 +17,6 @@ use crate::{
     library::{Infallible, Mandatory, Nullable},
     version::Version,
 };
-use log::error;
-use std::{collections::HashSet, str::FromStr};
-use toml::Value;
 
 #[derive(Clone, Debug)]
 pub struct CallbackParameter {
@@ -25,7 +27,7 @@ pub struct CallbackParameter {
 pub type CallbackParameters = Vec<CallbackParameter>;
 
 impl Parse for CallbackParameter {
-    fn parse(toml: &Value, object_name: &str) -> Option<CallbackParameter> {
+    fn parse(toml: &Value, object_name: &str) -> Option<Self> {
         let ident = match Ident::parse(toml, object_name, "callback parameter") {
             Some(ident) => ident,
             None => {
@@ -36,17 +38,14 @@ impl Parse for CallbackParameter {
                 return None;
             }
         };
-        toml.check_unwanted(
-            &["nullable"],
-            &format!("callback parameter {}", object_name),
-        );
+        toml.check_unwanted(&["nullable"], &format!("callback parameter {object_name}"));
 
         let nullable = toml
             .lookup("nullable")
             .and_then(Value::as_bool)
             .map(Nullable);
 
-        Some(CallbackParameter { ident, nullable })
+        Some(Self { ident, nullable })
     }
 }
 
@@ -59,10 +58,10 @@ impl AsRef<Ident> for CallbackParameter {
 #[derive(Clone, Debug)]
 pub struct Parameter {
     pub ident: Ident,
-    //true - parameter don't changed in FFI function,
-    //false(default) - parameter can be changed in FFI function
+    // true - parameter don't changed in FFI function,
+    // false(default) - parameter can be changed in FFI function
     pub constant: bool,
-    pub move_: bool,
+    pub move_: Option<bool>,
     pub nullable: Option<Nullable>,
     pub mandatory: Option<Mandatory>,
     pub infallible: Option<Infallible>,
@@ -72,7 +71,7 @@ pub struct Parameter {
 }
 
 impl Parse for Parameter {
-    fn parse(toml: &Value, object_name: &str) -> Option<Parameter> {
+    fn parse(toml: &Value, object_name: &str) -> Option<Self> {
         let ident = match Ident::parse(toml, object_name, "function parameter") {
             Some(ident) => ident,
             None => {
@@ -96,17 +95,14 @@ impl Parse for Parameter {
                 "string_type",
                 "callback_parameter",
             ],
-            &format!("function parameter {}", object_name),
+            &format!("function parameter {object_name}"),
         );
 
         let constant = toml
             .lookup("const")
             .and_then(Value::as_bool)
             .unwrap_or(false);
-        let move_ = toml
-            .lookup("move")
-            .and_then(Value::as_bool)
-            .unwrap_or(false);
+        let move_ = toml.lookup("move").and_then(Value::as_bool);
         let nullable = toml
             .lookup("nullable")
             .and_then(Value::as_bool)
@@ -141,7 +137,7 @@ impl Parse for Parameter {
         let callback_parameters =
             CallbackParameters::parse(toml.lookup("callback_parameter"), object_name);
 
-        Some(Parameter {
+        Some(Self {
             ident,
             constant,
             move_,
@@ -176,9 +172,9 @@ pub struct Return {
 }
 
 impl Return {
-    pub fn parse(toml: Option<&Value>, object_name: &str) -> Return {
+    pub fn parse(toml: Option<&Value>, object_name: &str) -> Self {
         if toml.is_none() {
-            return Return {
+            return Self {
                 nullable: None,
                 mandatory: None,
                 infallible: None,
@@ -246,7 +242,7 @@ impl Return {
             );
         }
 
-        Return {
+        Self {
             nullable,
             mandatory,
             infallible,
@@ -259,7 +255,7 @@ impl Return {
     }
 }
 
-fn check_rename(rename: &Option<String>, object_name: &str, function_name: &Ident) -> bool {
+pub fn check_rename(rename: &Option<String>, object_name: &str, function_name: &Ident) -> bool {
     if let Some(rename) = rename {
         for c in &["\t", "\n", " "] {
             if rename.contains(c) {
@@ -298,7 +294,7 @@ pub struct Function {
 }
 
 impl Parse for Function {
-    fn parse(toml: &Value, object_name: &str) -> Option<Function> {
+    fn parse(toml: &Value, object_name: &str) -> Option<Self> {
         let ident = match Ident::parse(toml, object_name, "function") {
             Some(ident) => ident,
             None => {
@@ -333,7 +329,7 @@ impl Parse for Function {
                 "visibility",
                 "generate_doc",
             ],
-            &format!("function {}", object_name),
+            &format!("function {object_name}"),
         );
 
         let status = {
@@ -429,7 +425,7 @@ impl Parse for Function {
             .lookup("generate_doc")
             .and_then(Value::as_bool)
             .unwrap_or(true);
-        Some(Function {
+        Some(Self {
             ident,
             status,
             version,
