@@ -1,3 +1,12 @@
+use std::{
+    collections::{BTreeMap, HashSet},
+    str::FromStr,
+    sync::Arc,
+};
+
+use log::{error, warn};
+use toml::Value;
+
 use super::{
     child_properties::ChildProperties,
     constants::Constants,
@@ -7,6 +16,7 @@ use super::{
     members::Members,
     properties::Properties,
     signals::{Signal, Signals},
+    virtual_methods::VirtualMethods,
 };
 use crate::{
     analysis::{conversion_type::ConversionType, ref_mode},
@@ -18,36 +28,24 @@ use crate::{
     library::{self, Library, TypeId, MAIN_NAMESPACE},
     version::Version,
 };
-use log::{error, warn};
-use std::{
-    collections::{BTreeMap, HashSet},
-    str::FromStr,
-    sync::Arc,
-};
-use toml::Value;
 
-#[derive(Clone, Copy, Debug, Eq, PartialEq)]
+#[derive(Default, Clone, Copy, Debug, Eq, PartialEq)]
 pub enum GStatus {
     Manual,
     Generate,
+    #[default]
     Ignore,
 }
 
 impl GStatus {
     pub fn ignored(self) -> bool {
-        self == GStatus::Ignore
+        self == Self::Ignore
     }
     pub fn manual(self) -> bool {
-        self == GStatus::Manual
+        self == Self::Manual
     }
     pub fn need_generate(self) -> bool {
-        self == GStatus::Generate
-    }
-}
-
-impl Default for GStatus {
-    fn default() -> GStatus {
-        GStatus::Ignore
+        self == Self::Generate
     }
 }
 
@@ -55,10 +53,10 @@ impl FromStr for GStatus {
     type Err = String;
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         match s {
-            "manual" => Ok(GStatus::Manual),
-            "generate" => Ok(GStatus::Generate),
-            "ignore" => Ok(GStatus::Ignore),
-            e => Err(format!("Wrong object status: \"{}\"", e)),
+            "manual" => Ok(Self::Manual),
+            "generate" => Ok(Self::Generate),
+            "ignore" => Ok(Self::Ignore),
+            e => Err(format!("Wrong object status: \"{e}\"")),
         }
     }
 }
@@ -68,6 +66,7 @@ impl FromStr for GStatus {
 pub struct GObject {
     pub name: String,
     pub functions: Functions,
+    pub virtual_methods: VirtualMethods,
     pub constants: Constants,
     pub signals: Signals,
     pub members: Members,
@@ -80,6 +79,7 @@ pub struct GObject {
     pub type_id: Option<TypeId>,
     pub final_type: Option<bool>,
     pub fundamental_type: Option<bool>,
+    pub exhaustive: bool,
     pub trait_name: Option<String>,
     pub child_properties: Option<ChildProperties>,
     pub concurrency: library::Concurrency,
@@ -106,6 +106,7 @@ impl Default for GObject {
         GObject {
             name: "Default".into(),
             functions: Functions::new(),
+            virtual_methods: VirtualMethods::new(),
             constants: Constants::new(),
             signals: Signals::new(),
             members: Members::new(),
@@ -118,6 +119,7 @@ impl Default for GObject {
             type_id: None,
             final_type: None,
             fundamental_type: None,
+            exhaustive: false,
             trait_name: None,
             child_properties: None,
             concurrency: Default::default(),
@@ -141,7 +143,7 @@ impl Default for GObject {
     }
 }
 
-//TODO: ?change to HashMap<String, GStatus>
+// TODO: ?change to HashMap<String, GStatus>
 pub type GObjects = BTreeMap<String, GObject>;
 
 pub fn parse_toml(
@@ -165,19 +167,6 @@ pub fn parse_toml(
     objects
 }
 
-fn ref_mode_from_str(ref_mode: &str) -> Option<ref_mode::RefMode> {
-    use crate::analysis::ref_mode::RefMode::*;
-
-    match ref_mode {
-        "none" => Some(None),
-        "ref" => Some(ByRef),
-        "ref-mut" => Some(ByRefMut),
-        "ref-immut" => Some(ByRefImmut),
-        "ref-fake" => Some(ByRefFake),
-        _ => Option::None,
-    }
-}
-
 pub fn parse_conversion_type(toml: Option<&Value>, object_name: &str) -> Option<ConversionType> {
     use crate::analysis::conversion_type::ConversionType::*;
 
@@ -274,6 +263,7 @@ fn parse_object(
             "child_type",
             "final_type",
             "fundamental_type",
+            "exhaustive",
             "trait",
             "trait_name",
             "cfg_condition",
@@ -292,7 +282,7 @@ fn parse_object(
             "default_value",
             "generate_doc",
         ],
-        &format!("object {}", name),
+        &format!("object {name}"),
     );
 
     let status = match toml_object.lookup("status") {
@@ -307,7 +297,17 @@ fn parse_object(
     let mut function_names = HashSet::new();
     for f in &functions {
         if let Ident::Name(name) = &f.ident {
-            assert!(function_names.insert(name), "{} already defined!", name);
+            assert!(function_names.insert(name), "{name} already defined!");
+        }
+    }
+    let virtual_methods = VirtualMethods::parse(toml_object.lookup("virtual_method"), &name);
+    let mut virtual_methods_names = HashSet::new();
+    for f in &virtual_methods {
+        if let Ident::Name(name) = &f.ident {
+            assert!(
+                virtual_methods_names.insert(name),
+                "{name} already defined!"
+            );
         }
     }
 
@@ -348,6 +348,10 @@ fn parse_object(
     let fundamental_type = toml_object
         .lookup("fundamental_type")
         .and_then(Value::as_bool);
+    let exhaustive = toml_object
+        .lookup("exhaustive")
+        .and_then(Value::as_bool)
+        .unwrap_or(false);
     let trait_name = toml_object
         .lookup("trait_name")
         .and_then(Value::as_str)
@@ -360,7 +364,7 @@ fn parse_object(
     let ref_mode = toml_object
         .lookup("ref_mode")
         .and_then(Value::as_str)
-        .and_then(ref_mode_from_str);
+        .and_then(|v| v.parse().ok());
     let conversion_type = parse_conversion_type(toml_object.lookup("conversion_type"), &name);
     let child_properties = ChildProperties::parse(toml_object, &name);
     let must_use = toml_object
@@ -429,7 +433,7 @@ fn parse_object(
     let visibility = toml_object
         .lookup("visibility")
         .and_then(Value::as_str)
-        .map(std::str::FromStr::from_str)
+        .map(|v| v.parse())
         .transpose();
     if let Err(ref err) = visibility {
         error!("{}", err);
@@ -488,6 +492,7 @@ fn parse_object(
     GObject {
         name,
         functions,
+        virtual_methods,
         constants,
         signals,
         members,
@@ -500,6 +505,7 @@ fn parse_object(
         type_id: None,
         final_type,
         fundamental_type,
+        exhaustive,
         trait_name,
         child_properties,
         concurrency,
@@ -553,7 +559,7 @@ fn parse_status_shorthand(
     generate_builder: bool,
     trust_return_value_nullability: bool,
 ) {
-    let option_name = format!("options.{:?}", status).to_ascii_lowercase();
+    let option_name = format!("options.{status:?}").to_ascii_lowercase();
     if let Some(a) = toml.lookup(&option_name).map(|a| a.as_array().unwrap()) {
         for name in a.iter().map(|s| s.as_str().unwrap()) {
             match objects.get(name) {
@@ -571,7 +577,7 @@ fn parse_status_shorthand(
                         },
                     );
                 }
-                Some(_) => panic!("Bad name in {}: {} already defined", option_name, name),
+                Some(_) => panic!("Bad name in {option_name}: {name} already defined"),
             }
         }
     }
@@ -583,7 +589,7 @@ pub fn resolve_type_ids(objects: &mut GObjects, library: &Library) {
 
     for (name, object) in objects.iter_mut() {
         let type_id = library.find_type(0, name);
-        if type_id.is_none() && name != &global_functions_name {
+        if type_id.is_none() && name != &global_functions_name && object.status != GStatus::Ignore {
             warn!("Configured object `{}` missing from the library", name);
         } else if object.generate_builder {
             if let Some(type_id) = type_id {
@@ -605,8 +611,7 @@ pub fn resolve_type_ids(objects: &mut GObjects, library: &Library) {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::analysis::conversion_type::ConversionType;
-    use crate::library::Concurrency;
+    use crate::{analysis::conversion_type::ConversionType, library::Concurrency};
 
     fn toml(input: &str) -> ::toml::Value {
         let value = ::toml::from_str(input);
