@@ -1,3 +1,7 @@
+use std::collections::HashMap;
+
+use log::{error, info};
+
 use crate::{
     analysis::types::IsIncomplete,
     config::{
@@ -10,8 +14,6 @@ use crate::{
     parser::is_empty_c_type,
     traits::MaybeRefAs,
 };
-use log::{error, info};
-use std::collections::HashMap;
 
 impl Namespace {
     fn unresolved(&self) -> Vec<&str> {
@@ -60,15 +62,11 @@ impl Library {
                 self.namespace(id)
                     .unresolved()
                     .into_iter()
-                    .map(move |s| format!("{}.{}", name, s))
+                    .map(move |s| format!("{name}.{s}"))
             })
             .collect();
 
-        assert!(
-            list.is_empty(),
-            "Incomplete library, unresolved: {:?}",
-            list
-        );
+        assert!(list.is_empty(), "Incomplete library, unresolved: {list:?}");
     }
 
     fn fill_empty_signals_c_types(&mut self) {
@@ -98,7 +96,7 @@ impl Library {
         let mut c_types = DetectedCTypes::new();
         for (ns_id, ns) in self.namespaces.iter().enumerate() {
             for (id, type_) in ns.types.iter().enumerate() {
-                let type_ = type_.as_ref().unwrap(); //Always contains something
+                let type_ = type_.as_ref().unwrap(); // Always contains something
                 let tid = TypeId {
                     ns_id: ns_id as u16,
                     id: id as u32,
@@ -173,7 +171,7 @@ impl Library {
         let type_ = self.type_(tid);
         type_.get_glib_name().map(|glib_name| {
             if self.is_referenced_type(type_) {
-                format!("{}*", glib_name)
+                format!("{glib_name}*")
             } else {
                 glib_name.to_string()
             }
@@ -195,7 +193,7 @@ impl Library {
 
         for (ns_id, ns) in self.namespaces.iter().enumerate() {
             for type_ in &ns.types {
-                let type_ = type_.as_ref().unwrap(); //Always contains something
+                let type_ = type_.as_ref().unwrap(); // Always contains something
 
                 if let Type::Record(record) = type_ {
                     if let Some(ref struct_for) = record.gtype_struct_for {
@@ -228,7 +226,7 @@ impl Library {
     fn correlate_class_structs(&self) {
         for (ns_id, ns) in self.namespaces.iter().enumerate() {
             for type_ in &ns.types {
-                let type_ = type_.as_ref().unwrap(); //Always contains something
+                let type_ = type_.as_ref().unwrap(); // Always contains something
                 let name;
                 let type_struct;
                 let c_class_type;
@@ -255,9 +253,7 @@ impl Library {
                     let type_struct_tid = self.find_type(ns_id as u16, type_struct);
                     assert!(
                         type_struct_tid.is_some(),
-                        "\"{}\" has glib:type-struct=\"{}\" but there is no such record",
-                        name,
-                        type_struct
+                        "\"{name}\" has glib:type-struct=\"{type_struct}\" but there is no such record"
                     );
 
                     let type_struct_type = self.type_(type_struct_tid.unwrap());
@@ -279,14 +275,11 @@ impl Library {
                         }
                     } else {
                         panic!(
-                            "Element with name=\"{}\" should be a record but it isn't",
-                            type_struct
+                            "Element with name=\"{type_struct}\" should be a record but it isn't"
                         );
                     }
                 } else if let Some(c) = c_class_type {
-                    panic!("\"{}\" has no glib:type-struct but there is an element with glib:is-gtype-struct-for=\"{}\"",
-                           name,
-                           c);
+                    panic!("\"{name}\" has no glib:type-struct but there is an element with glib:is-gtype-struct-for=\"{c}\"");
                 }
                 // else both type_struct and c_class_type are None,
                 // and that's fine because they don't reference each
@@ -303,7 +296,7 @@ impl Library {
         let mut actions: Vec<(TypeId, usize, Action)> = Vec::new();
         for (ns_id, ns) in self.namespaces.iter().enumerate() {
             for (id, type_) in ns.types.iter().enumerate() {
-                let type_ = type_.as_ref().unwrap(); //Always contains something
+                let type_ = type_.as_ref().unwrap(); // Always contains something
                 let tid = TypeId {
                     ns_id: ns_id as u16,
                     id: id as u32,
@@ -356,7 +349,8 @@ impl Library {
                 | Type::Record(Record { name, fields, .. })
                 | Type::Union(Union { name, fields, .. }) => match action {
                     Action::SetCType(c_type) => {
-                        // Don't be verbose when internal fields such as padding don't provide a c-type
+                        // Don't be verbose when internal fields such as padding don't provide a
+                        // c-type
                         if !ignore_missing_ctype.contains(&fields[fid].name.as_str()) {
                             warn_main!(
                                 tid,
@@ -425,17 +419,19 @@ impl Library {
 
     fn mark_final_types(&mut self, config: &Config) {
         // Here we mark all class types as final types if configured so in the config or
-        // otherwise if there is no public class struct for the type or the instance struct
-        // has no fields (i.e. is not known!), and there are no known subtypes.
+        // otherwise if there is no public class struct for the type or the instance
+        // struct has no fields (i.e. is not known!), and there are no known
+        // subtypes.
         //
         // Final types can't have any subclasses and we handle them slightly different
         // for that reason.
-        // FIXME: without class_hierarchy this function O(n2) due inner loop in `has_subtypes`
+        // FIXME: without class_hierarchy this function O(n2) due inner loop in
+        // `has_subtypes`
         let mut final_types: Vec<TypeId> = Vec::new();
 
         for (ns_id, ns) in self.namespaces.iter().enumerate() {
             for (id, type_) in ns.types.iter().enumerate() {
-                let type_ = type_.as_ref().unwrap(); //Always contains something
+                let type_ = type_.as_ref().unwrap(); // Always contains something
 
                 if let Type::Class(klass) = type_ {
                     let tid = TypeId {
@@ -495,7 +491,7 @@ impl Library {
         let mut error_domains = vec![];
         for (ns_id, ns) in self.namespaces.iter().enumerate() {
             'next_enum: for (id, type_) in ns.types.iter().enumerate() {
-                let type_ = type_.as_ref().unwrap(); //Always contains something
+                let type_ = type_.as_ref().unwrap(); // Always contains something
                 let enum_tid = TypeId {
                     ns_id: ns_id as u16,
                     id: id as u32,
@@ -507,14 +503,14 @@ impl Library {
 
                         let mut function_candidates = vec![domain.clone()];
                         if !domain.ends_with("_quark") {
-                            function_candidates.push(format!("{}_quark", domain));
+                            function_candidates.push(format!("{domain}_quark"));
                         }
                         if !domain.ends_with("_error_quark") {
                             if domain.ends_with("_quark") {
                                 function_candidates
                                     .push(format!("{}_error_quark", &domain[..(domain.len() - 6)]));
                             } else {
-                                function_candidates.push(format!("{}_error_quark", domain));
+                                function_candidates.push(format!("{domain}_error_quark"));
                             }
                         }
                         if let Some(domain) = domain.strip_suffix("_error_quark") {
@@ -540,7 +536,7 @@ impl Library {
 
                         // Quadratic in number of types...
                         for (id, type_) in ns.types.iter().enumerate() {
-                            let type_ = type_.as_ref().unwrap(); //Always contains something
+                            let type_ = type_.as_ref().unwrap(); // Always contains something
                             let domain_tid = TypeId {
                                 ns_id: ns_id as u16,
                                 id: id as u32,
